@@ -40,11 +40,11 @@ pub enum Card {
     ScalarFloat(FloatNode),
     StringLiteral(StringNode),
     CallNative(Box<CallNode>),
-    IfTrue(CardId),
-    IfFalse(CardId),
+    IfTrue(Box<Card>),
+    IfFalse(Box<Card>),
     IfElse {
-        then: CardId,
-        r#else: CardId,
+        then: Box<Card>,
+        r#else: Box<Card>,
     },
     Jump(LaneNode),
     SetGlobalVar(VarNode),
@@ -57,11 +57,7 @@ pub enum Card {
         lane: LaneNode,
     },
     /// Single card that decomposes into multiple cards
-    // TODO: move into struct and store a Box<CompositeCard> to reduce Card size?
-    CompositeCard {
-        name: Option<String>,
-        cards: Vec<CardId>,
-    },
+    CompositeCard(Box<CompositeCard>),
     /// Does nothing
     Noop,
 }
@@ -106,9 +102,7 @@ impl Card {
             Card::GetProperty => "GetProperty",
             Card::SetProperty => "SetProperty",
             Card::ForEach { .. } => "ForEach",
-            Card::CompositeCard { name, .. } => {
-                name.as_ref().map(|x| x.as_str()).unwrap_or("CompositeCard")
-            }
+            Card::CompositeCard(c) => c.name.as_deref().unwrap_or("CompositeCard"),
             Card::Noop => "No-op",
         }
     }
@@ -207,6 +201,26 @@ impl Card {
             | Instruction::Pass => {}
         };
     }
+
+    pub fn as_composite_card(&self) -> Option<&CompositeCard> {
+        if let Self::CompositeCard(v) = self {
+            Some(v)
+        } else {
+            None
+        }
+    }
+
+    pub fn as_composite_card_mut(&mut self) -> Option<&mut CompositeCard> {
+        if let Self::CompositeCard(v) = self {
+            Some(v)
+        } else {
+            None
+        }
+    }
+
+    pub fn composite_card(name: Option<String>, cards: Vec<Card>) -> Self {
+        Self::CompositeCard(Box::new(CompositeCard { name, cards }))
+    }
 }
 
 #[derive(Debug, Clone, Default, Copy)]
@@ -263,3 +277,10 @@ impl<'a> From<&'a LaneNode> for Handle {
         Handle::from_str(ln.0.as_str()).unwrap()
     }
 }
+
+#[derive(Debug, Clone, Default)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+pub struct CompositeCard {
+    pub name: Option<String>,
+    pub cards: Vec<Card>,
+}
