@@ -5,7 +5,7 @@ use crate::compiler::Lane;
 use crate::prelude::CompilationErrorPayload;
 use smallvec::SmallVec;
 use std::collections::hash_map::DefaultHasher;
-use std::collections::{BTreeMap, BTreeSet, HashMap};
+use std::collections::{BTreeMap, BTreeSet};
 use std::hash::Hasher;
 use std::rc::Rc;
 use thiserror::Error;
@@ -24,33 +24,6 @@ pub enum IntoStreamError {
 pub type CaoProgram = Module;
 pub type CaoIdentifier = String;
 
-pub type ModuleCards = HashMap<CardId, super::Card>;
-
-#[derive(Debug, Clone, Default, Copy, PartialEq, PartialOrd, Ord, Eq, Hash)]
-#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
-pub struct CardId(pub u64);
-
-impl std::fmt::Display for CardId {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}", self.0)
-    }
-}
-
-impl From<u64> for CardId {
-    fn from(i: u64) -> Self {
-        CardId(i)
-    }
-}
-
-impl std::str::FromStr for CardId {
-    type Err = <u64 as std::str::FromStr>::Err;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        let i = u64::from_str(s)?;
-        Ok(CardId(i))
-    }
-}
-
 #[derive(Debug, Clone, Default)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub struct Module {
@@ -60,16 +33,125 @@ pub struct Module {
     ///
     /// e.g. importing `foo.bar` allows you to use a `Jump("bar")` [[Card]]
     pub imports: BTreeSet<CaoIdentifier>,
+}
 
-    /// This field holds the actual card instances used by this module.
-    /// This representation is more convenient for reordering/moving cards between lanes.
-    /// While we could use [module_name, lane_name, index] as a unique index, it doesn't work for
-    /// inline cards in the case of CompositeCards. Instead of "edge case poisoning" the indexing
-    /// the author has elected to use a unique integer based index for all cards
-    pub cards: ModuleCards,
+/// Uniquely index a card in a module
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+pub struct CardIndex<'a> {
+    pub lane: &'a str,
+    pub card_index: LaneCardIndex,
+}
+
+impl<'a> CardIndex<'a> {
+    pub fn new(lane: &'a str, card_index: usize) -> Self {
+        Self {
+            lane,
+            card_index: LaneCardIndex::new(card_index),
+        }
+    }
+
+    /// pushes a new sub-index to the bottom layer
+    #[must_use]
+    pub fn with_sub_index(mut self, card_index: usize) -> Self {
+        let mut last = &mut self.card_index.sub_card_index;
+        while let Some(x) = last {
+            if x.sub_card_index.is_none() {
+                x.sub_card_index = Some(Box::new(LaneCardIndex::new(card_index)));
+                return self;
+            }
+            last = &mut x.sub_card_index;
+        }
+        // else
+        self.card_index.sub_card_index = Some(Box::new(LaneCardIndex::new(card_index)));
+        self
+    }
+}
+
+#[derive(Debug, Clone, Default, PartialEq, Eq, Hash)]
+pub struct LaneCardIndex {
+    pub card_index: usize,
+    pub sub_card_index: Option<Box<LaneCardIndex>>,
+}
+
+impl LaneCardIndex {
+    #[must_use]
+    pub fn new(card_index: usize) -> Self {
+        Self {
+            card_index,
+            sub_card_index: None,
+        }
+    }
+
+    #[must_use]
+    pub fn with_sub_index(mut self, card_index: usize) -> Self {
+        self.sub_card_index = Some(Box::new(Self {
+            card_index,
+            sub_card_index: None,
+        }));
+        self
+    }
+}
+
+#[derive(Debug, Clone, Error)]
+pub enum CardFetchError {
+    #[error("Lane not found")]
+    LaneNotFound,
+    #[error("Card not found")]
+    CardNotFound,
+    #[error("The card has no nested lanes, but the index tried to fetch one")]
+    NoSubLane,
 }
 
 impl Module {
+    pub fn get_card_mut<'a>(
+        &'a mut self,
+        idx: &CardIndex,
+    ) -> Result<&'a mut super::Card, CardFetchError> {
+        let lane = self
+            .lanes
+            .get_mut(idx.lane)
+            .ok_or(CardFetchError::LaneNotFound)?;
+        let mut card = lane
+            .cards
+            .get_mut(idx.card_index.card_index)
+            .ok_or(CardFetchError::CardNotFound)?;
+
+        let mut card_idx = idx.card_index.sub_card_index.as_ref();
+        while let Some(sub_card_idx) = card_idx {
+            card = card
+                .as_composite_card_mut()
+                .ok_or(CardFetchError::NoSubLane)?
+                .cards
+                .get_mut(sub_card_idx.card_index)
+                .ok_or(CardFetchError::CardNotFound)?;
+            card_idx = sub_card_idx.sub_card_index.as_ref();
+        }
+        Ok(card)
+    }
+
+    pub fn get_card<'a>(&'a self, idx: &CardIndex) -> Result<&'a super::Card, CardFetchError> {
+        let lane = self
+            .lanes
+            .get(idx.lane)
+            .ok_or(CardFetchError::LaneNotFound)?;
+        let mut card = lane
+            .cards
+            .get(idx.card_index.card_index)
+            .ok_or(CardFetchError::CardNotFound)?;
+
+        let mut card_idx = idx.card_index.sub_card_index.as_ref();
+        while let Some(sub_card_idx) = card_idx {
+            card = card
+                .as_composite_card()
+                .ok_or(CardFetchError::NoSubLane)?
+                .cards
+                .get(sub_card_idx.card_index)
+                .ok_or(CardFetchError::CardNotFound)?;
+            card_idx = sub_card_idx.sub_card_index.as_ref();
+        }
+        Ok(card)
+    }
+
     /// flatten this program into a vec of lanes
     pub(crate) fn into_ir_stream(
         mut self,
@@ -83,7 +165,7 @@ impl Module {
             .ok_or(CompilationErrorPayload::NoMain)?;
 
         let imports = self.execute_imports()?;
-        let first_fn = lane_to_compiled_lane(&self.cards, &first_fn, &["main"], Rc::new(imports));
+        let first_fn = lane_to_compiled_lane(&first_fn, &["main"], Rc::new(imports));
         let mut result = vec![first_fn];
         result.reserve(self.lanes.len() * self.submodules.len() * 2); // just some dumb heuristic
 
@@ -136,8 +218,8 @@ fn hash_module(hasher: &mut impl Hasher, module: &Module) {
 }
 
 fn hash_lane(hasher: &mut impl Hasher, lane: &Lane) {
-    for card_id in lane.cards.iter() {
-        hasher.write_u64(card_id.0);
+    for card in lane.cards.iter() {
+        hasher.write(card.name().as_bytes());
     }
 }
 
@@ -166,23 +248,13 @@ fn flatten_module<'a>(
             return Err(CompilationErrorPayload::BadLaneName(name.to_string()));
         }
         namespace.push(name.as_ref());
-        out.push(lane_to_compiled_lane(
-            &module.cards,
-            lane,
-            namespace,
-            Rc::clone(&imports),
-        ));
+        out.push(lane_to_compiled_lane(lane, namespace, Rc::clone(&imports)));
         namespace.pop();
     }
     Ok(())
 }
 
-fn lane_to_compiled_lane(
-    cards: &ModuleCards,
-    lane: &Lane,
-    namespace: &[&str],
-    imports: Rc<Imports>,
-) -> LaneIr {
+fn lane_to_compiled_lane(lane: &Lane, namespace: &[&str], imports: Rc<Imports>) -> LaneIr {
     assert!(
         !namespace.is_empty(),
         "Assume that lane name is the last entry in namespace"
@@ -193,7 +265,6 @@ fn lane_to_compiled_lane(
         arguments: lane.arguments.clone().into_boxed_slice(),
         cards: lane.cards.clone().into_boxed_slice(),
         imports,
-        card_impls: cards.clone(), // TODO can we get rid of this copy?
         ..Default::default()
     };
     cl.namespace.extend(
@@ -240,27 +311,25 @@ mod tests {
         use crate::compiler::{Card, StringNode};
 
         let mut lanes = BTreeMap::new();
-        lanes.insert("main".into(), Lane::default().with_card(1));
-        let cards = [
-            (
-                1.into(),
-                Card::CompositeCard {
+        lanes.insert(
+            "main".into(),
+            Lane::default().with_card(Card::CompositeCard(Box::new(
+                crate::compiler::CompositeCard {
                     name: "triplepog".to_string().into(),
-                    cards: vec![2.into(), 2.into(), 2.into()],
+                    cards: vec![
+                        Card::StringLiteral(StringNode("poggers".to_owned())),
+                        Card::StringLiteral(StringNode("poggers".to_owned())),
+                        Card::StringLiteral(StringNode("poggers".to_owned())),
+                    ],
                 },
-            ),
-            (
-                2.into(),
-                Card::StringLiteral(StringNode("poggers".to_owned())),
-            ),
-        ]
-        .into();
-        Module {
+            ))),
+        );
+        let default_prog = CaoProgram {
             imports: Default::default(),
             submodules: Default::default(),
             lanes,
-            cards,
-        }
+        };
+        default_prog
     }
 
     #[test]
@@ -270,8 +339,6 @@ mod tests {
         let pl = serde_json::to_string_pretty(&default_prog).unwrap();
 
         let _prog: Module = serde_json::from_str(&pl).unwrap();
-
-        assert_eq!(default_prog.cards.len(), _prog.cards.len());
     }
 
     #[test]
@@ -281,18 +348,74 @@ mod tests {
         {
             "submodules": {},
             "imports": [],
-            "cards": {
-                "1": {"Jump": "42" }
-            },
             "lanes": {"main": {
                 "name": "main",
                 "arguments": [],
-                "cards": [ 1 ]
+                "cards": [ {"Jump": "42" } ]
             }}
         }
 "#;
         let _prog: Module = serde_json::from_str(&json).unwrap();
+    }
 
-        assert_eq!(1, _prog.cards.len());
+    #[test]
+    fn module_card_fetch_test() {
+        let m = prog();
+
+        let comp_card = m
+            .get_card(&CardIndex {
+                lane: "main",
+                card_index: LaneCardIndex {
+                    card_index: 0,
+                    sub_card_index: None,
+                },
+            })
+            .expect("failed to fetch card");
+
+        assert!(matches!(
+            comp_card,
+            super::super::Card::CompositeCard { .. }
+        ));
+
+        let nested_card = m
+            .get_card(&CardIndex {
+                lane: "main",
+                card_index: LaneCardIndex {
+                    card_index: 0,
+                    sub_card_index: Some(Box::new(LaneCardIndex {
+                        card_index: 1,
+                        sub_card_index: None,
+                    })),
+                },
+            })
+            .expect("failed to fetch nested card");
+
+        assert!(matches!(nested_card, super::super::Card::StringLiteral(_)));
+    }
+
+    #[test]
+    fn construct_sub_index_test() {
+        let index = CardIndex::new("pog", 0);
+
+        assert!(index.card_index.sub_card_index.is_none());
+
+        let index = index.with_sub_index(1);
+
+        assert_eq!(
+            index.card_index.sub_card_index.as_ref().unwrap().card_index,
+            1
+        );
+
+        let index = index.with_sub_index(2);
+        assert_eq!(
+            index
+                .card_index
+                .sub_card_index
+                .unwrap()
+                .sub_card_index
+                .unwrap()
+                .card_index,
+            2
+        );
     }
 }
